/*
 * init code specific for ICNova Base
 *
 * Copyright (C) 2007-2008 Benjamin Tietz <benjamin.tietz@in-circuit.de>
 *
 * based on atstk1002.c from ATMEL
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
//?#include <linux/bootmem.h>
#include <linux/clk.h>
#include <linux/etherdevice.h>
#include <linux/i2c-gpio.h> //??+
#include <linux/init.h>
#include <linux/linkage.h>
#include <linux/platform_device.h>
#include <linux/types.h>
#include <linux/leds.h>  //??+
#include <linux/spi/spi.h> //??+

#include <linux/kernel.h> //?
#include <linux/string.h> //?

#include <asm/io.h>
#include <asm/setup.h>

#include <asm/arch/at32ap700x.h>
#include <asm/arch/board.h>
#include <asm/arch/init.h>
#include <asm/arch/portmux.h>

#include "icnova.h"
#include "led.h"
#include "btn.h"
//#include "mac.h"

//#include "pwm.h"

#define GPIO_AC_OUTPUT_SELECT	GPIO_PIN_PB(15)
#define GPIO_AC_RESET		GPIO_PIN_PB(19)

/* Oscillator frequencies. These are board-specific */
unsigned long at32_board_osc_rates[3] = {
	[0] = 32768,	/* 32.768 kHz on RTC osc */
	[1] = 20000000,	/* 20 MHz on osc0 */
	[2] = 12000000,	/* 12 MHz on osc1 */
};

/* Initialized by bootloader-specific startup code. */
struct tag *bootloader_tags __initdata;

struct eth_addr {
	u8 addr[6];
};
static struct eth_addr __initdata hw_addr[1];
static struct eth_platform_data __initdata eth_data[1];

static struct mci_platform_data __initdata mci0_data = {
  .detect_pin = GPIO_PIN_NONE,
  .wp_pin = GPIO_PIN_NONE,
};

//static struct i2c_board_info icnova_i2c[] = {
//};

/*
 * The next two functions should go away as the boot loader is
 * supposed to initialize the macb address registers with a valid
 * ethernet address. But we need to keep it around for a while until
 * we can be reasonably sure the boot loader does this.
 *
 * The phy_id is ignored as the driver will probe for it.
 */
static int __init parse_tag_ethernet(struct tag *tag)
{
	int i;

	i = tag->u.ethernet.mac_index;
	if (i < ARRAY_SIZE(hw_addr))
		memcpy(hw_addr[i].addr, tag->u.ethernet.hw_address,
		       sizeof(hw_addr[i].addr));

	return 0;
}
__tagtable(ATAG_ETHERNET, parse_tag_ethernet);

static void __init set_hw_addr(struct platform_device *pdev)
{
	struct resource *res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	const u8 *addr;
	void __iomem *regs;
	struct clk *pclk;

	if (!res)
		return;
	if (pdev->id >= ARRAY_SIZE(hw_addr))
		return;

	addr = hw_addr[pdev->id].addr;
	if (!is_valid_ether_addr(addr))
		return;

	/*
	 * Since this is board-specific code, we'll cheat and use the
	 * physical address directly as we happen to know that it's
	 * the same as the virtual address.
	 */
	regs = (void __iomem __force *)res->start;
	pclk = clk_get(&pdev->dev, "pclk");
	if (!pclk)
		return;

	clk_enable(pclk);
	__raw_writel((addr[3] << 24) | (addr[2] << 16)
		     | (addr[1] << 8) | addr[0], regs + 0x98);
	__raw_writel((addr[5] << 8) | addr[4], regs + 0x9c);
	clk_disable(pclk);
	clk_put(pclk);
}



void __init setup_board(void)
{
#ifdef CONFIG_BOARD_ICNOVA_BASE_CP2102
	at32_map_usart(0, 1);	// USART 0: /dev/ttyS1, IF-to-USB-UART_Bridge
	at32_map_usart(1, 0);	// USART 1: /dev/ttyS0, CP2102
#else
	at32_map_usart(0, 0);	// USART 0: /dev/ttyS0, IF-to-USB-UART_Bridge
#endif
	//icnova_setup_irda(at32_add_device_usart(2));
	at32_setup_serial_console(0);

	//icnova_add_spi();
}

static int __init icnova_init(void)
{

	icnova_reserve_pins();

	at32_add_system_devices();

	at32_add_device_usart(0);
#ifdef CONFIG_BOARD_ICNOVA_BASE_CP2102
	at32_add_device_usart(1);
#endif

	//at32_add_device_pwm((1<<2));
	//icnova_setup_pwm(2,25000000); // 25MHz

	icnova_setup_leds();
	icnova_setup_buttons();

	set_hw_addr(at32_add_device_eth(0, &eth_data[0]));

	at32_add_device_twi(0, NULL, 0); // Nothing yet on I2C
	//at32_add_device_twi(0, icnova_i2c,ARRAY_SIZE(icnova_i2c));
	at32_add_device_usba(0, NULL);

	at32_add_device_mci(0, &mci0_data); /* MMC/SD */

//	at32_select_gpio(GPIO_AC_OUTPUT_SELECT,AT32_GPIOF_OUTPUT);
//	at32_select_gpio(GPIO_AC_RESET,AT32_GPIOF_OUTPUT);
//	at32_add_device_ac97c(0);

	return 0;
}
postcore_initcall(icnova_init);
