
#include <linux/leds.h>
#include <linux/platform_device.h>
#include <asm/arch/portmux.h>
#include <asm/arch/at32ap700x.h>

#define CONFIG_BOARD_ICNOVA_BASE  /* + wb */
#include "led.h"

static struct gpio_led icnova_led[] = {
#ifdef CONFIG_BOARD_ICNOVA_BASE
	{ 
		.name = "pwrled:red",
		.gpio = GPIO_PIN_PA(22),
		.active_low = 1,
	},
	{
		.name = "led1:green",
		.gpio = GPIO_PIN_PA(23),
		.default_trigger = "heartbeat",
	},
	{
		.name = "led2:green",
		.gpio = GPIO_PIN_PA(24),
	},
	{
		.name = "led3:green",
		.gpio = GPIO_PIN_PA(25),
	},
	{
		.name = "led4:green",
		.gpio = GPIO_PIN_PA(26),
	},
	{
		.name = "led5:green",
		.gpio = GPIO_PIN_PA(27),
	},
	{
		.name = "led6:green",
		.gpio = GPIO_PIN_PA(28),
	},
	{
		.name = "led7:green",
		.gpio = GPIO_PIN_PA(29),
	},
	{
		.name = "led8:green",
		.gpio = GPIO_PIN_PA(30),
	},
#else // big ICNova
	{
		.name = "pwrled:red",
		.gpio = GPIO_PIN_PA(25),
		.active_low = 1,
	},
	{
		.name = "led1:green",
		.gpio = GPIO_PIN_PC(18),
		.default_trigger = "heartbeat",
	},
#endif
};

static struct gpio_led_platform_data icnova_led_data = {
	.num_leds = ARRAY_SIZE(icnova_led),
	.leds = icnova_led,
};

static struct platform_device icnova_led_dev = {
	.name = "leds-gpio",
	.dev = {
		.platform_data = &icnova_led_data,
	},
};

void __init icnova_setup_leds(void) {
	unsigned int i;

	for(i=0;i<ARRAY_SIZE(icnova_led);i++)
		at32_select_gpio(icnova_led[i].gpio, AT32_GPIOF_OUTPUT);

	printk("ICNova: %lu Leds\n", ARRAY_SIZE(icnova_led));
	platform_device_register(&icnova_led_dev);
}
