/*
 * Abuse the PWM as an oscillator.
 *
 * licensed under the terms of the GPL
 * (C) 2007, Benjamin Tietz <benjamin@micronet24.de>
 */

#include <linux/atmel_pwm.h>
#include <linux/kernel.h>
#include <linux/errno.h>

#define PWM_NAME "icnova_pwm"
#define pwm_say(level, pwmnr, msg, arg...) \
	printk(level PWM_NAME "PWM%d: " msg "\n", pwmnr, arg)

// Set ups a PWM to oscillate at a specific frequency, with equal idle and 
// busy-times. If all goes wel the PWMs will be enabled and the actual
// frequency, otherwise an error-code will be returned.
int icnova_setup_pwm(int nr, unsigned long freq) {

	struct pwm_channel chan;
	int i;
	unsigned int pre, cprd;

	// pre-compute divisor and preset
	cprd = atmel_pwm_get_mck() / freq;
	if(cprd < 2) {
		pwm_say(KERN_CRIT, nr, "Can't set to %ldHz: frequency to high.",
				freq);
		return -EINVAL;
	}
	pre = 0;
	while(((cprd & 1) == 0) && (pre < 10)) {
		pre++;
		cprd >>= 1;
	}
	if(cprd == 1) {pre--; cprd<<=1;}

	// Allocate channel
	i = pwm_channel_alloc(nr, &chan);
	if(i<0) return i;

	pwm_channel_writel(&chan, PWM_CMR, pre);
	pwm_channel_writel(&chan, PWM_CDTY, cprd / 2);
	pwm_channel_writel(&chan, PWM_CPRD, cprd);
		
	pwm_channel_enable(&chan);

	pwm_say(KERN_NOTICE, nr, "Set to %ldHz",
			atmel_pwm_get_mck() / (cprd * (1<<pre)));
	return 0;
}
