/*
 * Abuse the PWM as an oscillator.
 *
 * licensed under the terms of the GPL
 * (C) 2007, Benjamin Tietz <benjamin@micronet24.de>
 */

#include <linux/atmel_pwm.h>
#include <linux/kernel.h>

#define PWM_NAME "icnova_pwm"
#define PWM_ERR(pwmnr, msg, arg...) \
	printk(KERN_CRIT PWM_NAME "PWM%d: " msg, pwmnr, arg)

#define icnova_pwm_get_global_mode(ch) __raw_readl(ch.regs & ~0x03FF)
#define icnova_pwm_set_global_mode(ch, val) __raw_writel(ch.regs & ~0x03FF, val)
#define icnova_pwm_check_clk(nr,div,pre,reg, cprd, cdty, ret) { \
	if(((reg & (0x00FF << (nr * 16))) == div) && \
			((reg & (0x0F00 << (nr * 16))) >= pre)) { \
			ret = nr; \
			cprd = 2 << ((reg & (0x0F00 << (nr * 16))) - pre + 1);\
			cdty = cprd/2; \
		} \
	} \
}

// Set ups a PWM to oscillate at a specific frequency, with equal idle and 
// busy-times. If all goes wel the PWMs will be enabled and the actual
// frequency, otherwise an error-code will be returned.
static int icnova_setup_pwm(int nr, unsigned long freq) {

	struct pwm_channel chan;
	int i, j, k;
	unsigned int div, pre;
	unsigned int cprd = 2, cdty = 1;

	// pre-compute divisor and preset
	div = atmel_pwm_get_mck() / (freq << 1);
	if(div == 0) {
		PWM_ERR(nr,"Can't set to %dHz: frequency to high.",freq);
		return -EINVAL;
	}
	pre = 0;
	while(((div & 1) == 0) && (pre < 10)) {
		pre++;
		div >>= 1;
	}

	// Allocate channel
	i = pwm_channel_alloc(nr, &chan);
	if(i<0) return i;

	if(div != 0) { // We need either CLKa or CLKb
		i = icnova_pwm_get_global_mode(chan);
		j = 2;
		for(k=0;k<j;k++)
			icnova_pwm_check_clk(k,div,pre,i,cprd, cdty, j);
		if(j==2) { // There was no matching CLK
			if((i & 0x0000FFFF) == 0) j = 0;
			else if((i & 0xFFFF0000) == 0) j = 1;
			else {
				PWM_ERR(nr, "No free clock");
				return -EBUSY;
			}
			icnova_pwm_set_global_mode(chan,
					(i&(0x0000FFFF << ((j==0?1:0)*16))) |
					((pre<<8|div) << (j*16)));
		}
		pwm_channel_writel(chan, PWM_CMR,(j?PWM_CPR_CLKA:PWM_CPR_CLKB));
	} else
		pwm_channel_writel(chan, PWM_CMR, pre);
	pwm_channel_writel(chan, PWM_CDTY, cdty);
	pwm_channel_writel(chan, PWM_CPRD, cprd);
		
	pwm_channel_enable(chan);
}
