/*
 * Code for initializeing the VGA-Port of ICNova
 *
 * Copyright (C) 2007 Benjamin Tietz, In-Circuit
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/screen_info.h>

#include <video/atmel_lcdc.h>

#include "vga.h"

static struct fb_videomode icnova_modes[] = {
#ifdef CONFIG_BOARD_ICNOVA_SINGLE_MODE_TFT
	.refresh	= CONFIG_BOARD_ICNOVA_VGA_VHZ,
	.xres		= CONFIG_BOARD_ICNOVA_VGA_XRES,
	.yres		= CONFIG_BOARD_ICNOVA_VGA_YRES,
	.pixclock	= CONFIG_BOARD_ICNOVA_VGA_PIXCLK,
	.left_margin	= CONFIG_BOARD_ICNOVA_VGA_LMARG,
	.right_margin	= CONFIG_BOARD_ICNOVA_VGA_RMARG,
	.upper_margin	= CONFIG_BOARD_ICNOVA_VGA_TMARG,
	.lower_margin	= CONFIG_BOARD_ICNOVA_VGA_BMARG,
	.hsync_len	= CONFIG_BOARD_ICNOVA_VGA_HSYNC,
	.vsync_len	= CONFIG_BOARD_ICNOVA_VGA_VSYNC,
	.sync		= 
#ifdef CONFIG_BOARD_ICNOVA_VGA_SYNCH
		FB_SYNC_HOR_HIGH_ACT | 
#endif
#ifdef CONFIG_BOARD_ICNOVA_VGA_SYNCV
		FB_SYNC_VERT_HIGH_ACT | 
#endif
#ifdef CONFIG_BOARD_ICNOVA_VGA_SYNCE
		FB_SYNC_EXT | 
#endif
#ifdef CONFIG_BOARD_ICNOVA_VGA_SYNCC
		FB_SYNC_COMP_HIGH_ACT | 
#endif
#ifdef CONFIG_BOARD_ICNOVA_VGA_SYNCB
		FB_SYNC_BROADCAST | 
#endif
#ifdef CONFIG_BOARD_ICNOVA_VGA_SYNCG
		FB_SYNC_ON_GREEN | 
#endif
		0,
	.vmode		= 
#ifdef CONFIG_BOARD_ICNOVA_VGA_INTERLACED
		FB_VMODE_INTERLACED,
#else
		FB_VMODE_NONINTERLACED,
#endif
	.flag		= 
#ifdef CONFIG_BOARD_ICNOVA_VGA_VESA
		FB_MODE_IS_VESA |
#endif
		0,
#else
	// Taken from vesa_modes
	/* 0 640x350-85 VESA */
//	{ "640x350-85", 85, 640, 350, 31746,  96, 32, 60, 32, 64, 3,
//	  FB_SYNC_HOR_HIGH_ACT, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA},
	/* 1 640x400-85 VESA */
//	{ "640x400-85", 85, 640, 400, 31746,  96, 32, 41, 01, 64, 3,
//	  FB_SYNC_VERT_HIGH_ACT, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 2 720x400-85 VESA */
//	{ "720x400-85", 85, 721, 400, 28169, 108, 36, 42, 01, 72, 3,
//	  FB_SYNC_VERT_HIGH_ACT, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 3 640x480-60 VESA */
	{ "640x480-60", 60, 640, 480, 39682,  58, 6, 33, 10, 96, 2, 
	  0, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 4 640x480-72 VESA */
//	{ "640x480-72", 72, 640, 480, 31746, 128, 24, 29, 9, 40, 2, 
//	  0, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 5 640x480-75 VESA */
//	{ "640x480-75", 75, 640, 480, 31746, 120, 16, 16, 01, 64, 3,
//	  0, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 6 640x480-85 VESA */
//	{ "640x480-85", 85, 640, 480, 27777, 80, 56, 25, 01, 56, 3,
//	  0, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 7 800x600-56 VESA */
	{ "800x600-56", 56, 800, 600, 27777, 128, 24, 22, 01, 72, 2,
	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	  FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 8 800x600-60 VESA */
	{ "800x600-60", 60, 800, 600, 25000, 88, 40, 23, 01, 128, 4,
	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	  FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 9 800x600-72 VESA */
	{ "800x600-72", 72, 800, 600, 20000, 64, 56, 23, 37, 120, 6,
	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	  FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 10 800x600-75 VESA */
	{ "800x600-75", 75, 800, 600, 20202, 160, 16, 21, 01, 80, 3,
	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	  FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 11 800x600-85 VESA */
	{ "800x600-85", 85, 800, 600, 17761, 152, 32, 27, 01, 64, 3,
	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	  FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
        /* 12 1024x768i-43 VESA */
//	{ "1024x768i-43", 43, 1024, 768, 22271, 56, 8, 41, 0, 176, 8,
//	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
//	  FB_VMODE_INTERLACED, FB_MODE_IS_VESA },
	/* 13 1024x768-60 VESA */
	{ "1024x768-60", 60, 1024, 768, 15384, 160, 24, 29, 3, 136, 6,
	  0, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 14 1024x768-70 VESA */
	{ "1024x768-70", 70, 1024, 768, 13333, 144, 24, 29, 3, 136, 6,
	  0, FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 15 1024x768-75 VESA */
	{ "1024x768-75", 75, 1024, 768, 12690, 176, 16, 28, 1, 96, 3,
	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	  FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
	/* 16 1024x768-85 VESA */
	{ "1024x768-85", 85, 1024, 768, 10582, 208, 48, 36, 1, 96, 3,
	  FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	  FB_VMODE_NONINTERLACED, FB_MODE_IS_VESA },
#endif
};

static struct fb_monspecs __initdata icnova_default_monspecs = {
	.manufacturer		= "VESA",
	.monitor		= "Generic VGA",
	.modedb			= icnova_modes,
	.modedb_len		= ARRAY_SIZE(icnova_modes),
	.hfmin			= 30000,
	.hfmax			= 100000,
	.vfmin			= 50,
	.vfmax			= 150,
	.dclkmax		= 100000000, // 100MHz
};

struct atmel_lcdfb_info __initdata icnova_lcdc_data = {
	.default_bpp		= 24,
	.default_dmacon		= ATMEL_LCDC_DMAEN | ATMEL_LCDC_DMA2DEN,
	.default_lcdcon2	= (ATMEL_LCDC_DISTYPE_TFT
				   | ATMEL_LCDC_INVCLK
				   | ATMEL_LCDC_CLKMOD_ALWAYSACTIVE
				   | ATMEL_LCDC_MEMOR_BIG),
	.default_monspecs	= &icnova_default_monspecs,
	.guard_time		= 2,
};

#if defined(CONFIG_VGA_CONSOLE) || defined(CONFIG_DUMMY_CONSOLE)
struct screen_info screen_info = {
	.orig_x			= 0,
	.orig_y			= 24,
	.ext_mem_k		= 0,
	.orig_video_page	= 0,
	.orig_video_mode	= 0,
	.orig_video_cols	= 80,
	.orig_video_ega_bx	= 0,
	.orig_video_lines	= 24,
	.orig_video_isVGA	= 1,
	.orig_video_points	= 16
};
EXPORT_SYMBOL(screen_info);
#endif

static int icnova_add_vga(void) {
	at32_add_device_lcdc(0, &icnova_lcdc_data, fbmem_start, fbmem_size);
	return 0;
}

device_initcall(icnova_add_vga);
